# Generated by Django 2.1.1 on 2018-10-04 12:19

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('bankreader', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30)),
                ('variable_symbol', models.CharField(max_length=10, unique=True)),
            ],
            options={
                'verbose_name': 'order',
                'verbose_name_plural': 'orders',
            },
        ),
        migrations.CreateModel(
            name='OrderPayment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=20, verbose_name='amount')),
                ('order', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='demoapp.Order')),
                ('transaction', models.OneToOneField(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='identified_order_payment', to='bankreader.Transaction')),
            ],
            options={
                'verbose_name': 'order payment',
                'verbose_name_plural': 'order payments',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from localflavor.generic.models import BICField, IBANField

from .readers import get_reader_choices, readers


class Account(models.Model):
    name = models.CharField(_('account name'), max_length=150, unique=True)
    iban = IBANField(_('IBAN'), blank=True, null=True)
    bic = BICField(_('BIC (SWIFT)'), blank=True, null=True)
    reader = models.CharField(
        _('account statement format'), blank=True, choices=get_reader_choices(), max_length=150, null=True
    )

    class Meta:
        verbose_name = _('account')
        verbose_name_plural = _('accounts')

    def __str__(self):
        return self.name

    def get_reader(self):
        return readers.get(self.reader)


class AccountStatement(models.Model):
    account = models.ForeignKey(
        Account,
        limit_choices_to={'reader__isnull': False},
        on_delete=models.CASCADE,
        related_name='account_statements',
        verbose_name=_('account'),
    )
    statement = models.CharField(_('statement'), max_length=256)
    from_date = models.DateField(_('from date'), editable=False)
    to_date = models.DateField(_('to date'), editable=False)

    class Meta:
        ordering = ('from_date',)
        verbose_name = _('account statement')
        verbose_name_plural = _('account statements')

    def __str__(self):
        return self.statement


class Transaction(models.Model):
    transaction_id = models.CharField(_('transaction id'), max_length=256)
    account_statement = models.ForeignKey(
        AccountStatement, on_delete=models.CASCADE, related_name='transactions', verbose_name=_('account statement')
    )
    account = models.ForeignKey(Account, on_delete=models.CASCADE, verbose_name=_('account'))
    entry_date = models.DateField(_('entry date'))
    accounted_date = models.DateField(_('accounted date'))
    remote_account_number = models.CharField(_('remote account number'), max_length=64)
    remote_account_name = models.CharField(_('remote account name'), max_length=128)
    amount = models.DecimalField(_('amount'), decimal_places=2, max_digits=20)
    variable_symbol = models.BigIntegerField(_('variable symbol'), default=0)
    constant_symbol = models.BigIntegerField(_('constant symbol'), default=0)
    specific_symbol = models.BigIntegerField(_('specific symbol'), default=0)
    sender_description = models.CharField(_('description for sender'), max_length=256)
    recipient_description = models.CharField(_('description for recipient'), max_length=256)

    class Meta:
        ordering = ('accounted_date',)
        unique_together = ('account', 'transaction_id')
        verbose_name = _('transaction')
        verbose_name_plural = _('transactions')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging

import django
from django import forms
from django.contrib import admin
from django.core.exceptions import ValidationError
from django.db import transaction
from django.db.models import Count
from django.db.utils import IntegrityError
from django.templatetags.static import static
from django.urls import reverse_lazy as reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from .models import Account, AccountStatement, Transaction

logger = logging.getLogger(__name__)


class AmountFieldListFilter(admin.FieldListFilter):
    def __init__(self, field, request, params, model, model_admin, field_path):
        self.lookup_kwarg_credit = '%s__gt' % field_path
        self.lookup_kwarg_debit = '%s__lt' % field_path
        self.lookup_val_credit = params.get(self.lookup_kwarg_credit)
        self.lookup_val_debit = params.get(self.lookup_kwarg_debit)
        super().__init__(field, request, params, model, model_admin, field_path)

    def expected_parameters(self):
        return [self.lookup_kwarg_credit, self.lookup_kwarg_debit]

    def choices(self, changelist):
        yield {
            'selected': self.lookup_val_credit is None and self.lookup_val_debit is None,
            'query_string': changelist.get_query_string(remove=[self.lookup_kwarg_credit, self.lookup_kwarg_debit]),
            'display': _('All'),
        }
        yield {
            'selected': self.lookup_val_credit == '0',
            'query_string': changelist.get_query_string({self.lookup_kwarg_credit: '0'}, [self.lookup_kwarg_debit]),
            'display': _('Credit transactions'),
        }
        yield {
            'selected': self.lookup_val_debit == '0',
            'query_string': changelist.get_query_string({self.lookup_kwarg_debit: '0'}, [self.lookup_kwarg_credit]),
            'display': _('Debit transactions'),
        }


class IdentifiedFieldListFilter(admin.RelatedFieldListFilter):
    def choices(self, changelist):
        yield {
            'selected': self.lookup_val_isnull is None,
            'query_string': changelist.get_query_string(remove=[self.lookup_kwarg_isnull]),
            'display': _('All'),
        }
        yield {
            'selected': self.lookup_val_isnull == 'True',
            'query_string': changelist.get_query_string({self.lookup_kwarg_isnull: 'True'}),
            'display': _('Not identified'),
        }
        yield {
            'selected': self.lookup_val_isnull == 'False',
            'query_string': changelist.get_query_string({self.lookup_kwarg_isnull: 'False'}),
            'display': _('Identified'),
        }


@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = ('name', 'iban', 'bic', 'account_statements_link')

    def get_queryset(self, request):
        return super().get_queryset(request).annotate(account_statements_count=Count('account_statements'))

    account_statement_changelist = reverse('admin:bankreader_accountstatement_changelist')

    @admin.display(description=_('account statements'), ordering='account_statements_count')
    def account_statements_link(self, obj):
        return mark_safe(
            '<a href="{url}?account__id__exact={account_id}">{count}</a>'.format(
                url=self.account_statement_changelist,
                account_id=obj.id,
                count=obj.account_statements_count,
            )
        )


class ReadOnlyMixin:
    # read only
    if django.VERSION > (2,):

        def has_change_permission(self, request, obj=None):
            return False

    else:

        def get_readonly_fields(self, request, obj=None):
            return tuple(f.name for f in self.model._meta.fields if not f.primary_key) if obj else ()


@admin.register(AccountStatement)
class AccountStatementAdmin(ReadOnlyMixin, admin.ModelAdmin):
    list_display = ('id', 'statement', 'account_name', 'from_date', 'to_date', 'transactions_link')
    list_filter = ('account',)

    def get_queryset(self, request):
        return (
            super().get_queryset(request).select_related('account').annotate(transactions_count=Count('transactions'))
        )

    @admin.display(description=_('account'), ordering='account__name')
    def account_name(self, obj):
        return obj.account.name

    transaction_changelist = reverse('admin:bankreader_transaction_changelist')

    @admin.display(description=_('transactions'), ordering='transactions_count')
    def transactions_link(self, obj):
        return mark_safe(
            '<a href="{url}?account_statement__id__exact={account_statement_id}">{count}</a>'.format(
                url=self.transaction_changelist,
                account_statement_id=obj.id,
                count=obj.transactions_count,
            )
        )

    class form(forms.ModelForm):
        statement = forms.FileField(label=_('account statement'))

        def clean(self):
            account = self.cleaned_data.get('account')
            statement = self.cleaned_data.get('statement')
            if account and account.reader and statement:
                reader = account.get_reader()
                try:
                    self.transactions_data = tuple(reader.read_archive(statement.file))
                except Exception:
                    msg = _('Failed to read transaction data in format {}.').format(reader.label)
                    logger.exception(msg)
                    raise ValidationError(msg)
                if not self.transactions_data:
                    raise ValidationError(_('The account statement doesn\'t contain any transaction data.'))
                self.instance.from_date = min(td['accounted_date'] for td in self.transactions_data)
                self.instance.to_date = max(td['accounted_date'] for td in self.transactions_data)

        def save(self, commit=True):
            if hasattr(self, 'transactions_data'):
                with transaction.atomic():
                    instance = super().save(commit)
                    instance.save()
                    for transaction_data in self.transactions_data:
                        try:
                            with transaction.atomic():
                                Transaction.objects.create(
                                    account=instance.account, account_statement=instance, **transaction_data
                                )
                        except IntegrityError:
                            pass
                return instance
            else:
                return super().save(commit)


class TransactionBaseAdmin(ReadOnlyMixin, admin.ModelAdmin):
    date_hierarchy = 'accounted_date'
    list_display = tuple(('statement' if f.name == 'account_statement' else f.name) for f in Transaction._meta.fields)[
        1:
    ] + tuple(r.name + '_link' for r in Transaction._meta.related_objects)
    list_filter = ('account_statement__account', ('amount', AmountFieldListFilter)) + tuple(
        (r.name, IdentifiedFieldListFilter) for r in Transaction._meta.related_objects
    )

    def get_queryset(self, request):
        qs = super().get_queryset(request).select_related('account', 'account_statement')
        for related_object in Transaction._meta.related_objects:
            qs = qs.select_related(related_object.name)
        return qs

    @admin.display(description=_('account statement'), ordering='account_statement__statement')
    def statement(self, obj):
        return obj.account_statement.statement

    def has_add_permission(self, request):
        return False


def _get_related_object_link(related_object):
    changelist_url = reverse(
        'admin:{}_{}_changelist'.format(
            related_object.related_model._meta.app_label,
            related_object.related_model._meta.model_name,
        )
    )
    add_url = reverse(
        'admin:{}_{}_add'.format(
            related_object.related_model._meta.app_label,
            related_object.related_model._meta.model_name,
        )
    )

    @admin.display(description=related_object.related_model._meta.verbose_name)
    def related_object_link(self, obj):
        try:
            related_obj = getattr(obj, related_object.name)
        except related_object.related_model.DoesNotExist:
            related_obj = None
        if related_obj:
            return format_html(
                '<a href="{changelist_url}?{remote_name}__id__exact={obj_id}">{text}</a>',
                changelist_url=changelist_url,
                remote_name=related_object.remote_field.name,
                obj_id=obj.id,
                text=getattr(obj, related_object.name),
            )
        else:
            return format_html(
                '<a href="{add_url}?{remote_name}={obj_id}" title="{title}">' '<img src="{icon}" alt="+"/></a>',
                add_url=add_url,
                remote_name=related_object.remote_field.name,
                obj_id=obj.id,
                title=_('add'),
                icon=static('admin/img/icon-addlink.svg'),
            )

    return related_object_link


TransactionAdmin = admin.register(Transaction)(
    type(
        "TransactionAdmin",
        (TransactionBaseAdmin,),
        {
            related_object.name + '_link': _get_related_object_link(related_object)
            for related_object in Transaction._meta.related_objects
        },
    )
)



import datetime
import decimal

from .base import BaseReader


class BestReader(BaseReader):
    label = 'Best'
    encoding = 'cp1250'

    def read_transactions(self, rows):
        for row in rows:
            if row[:2] == '52':
                yield {
                    'transaction_id': row[86:117].strip(),
                    'entry_date': datetime.datetime.strptime(row[167:175], '%Y%m%d').date(),
                    'accounted_date': datetime.datetime.strptime(row[175:183], '%Y%m%d').date(),
                    'remote_account_number': '%s-%s/%s'
                    % (
                        row[23:29],
                        row[29:39],
                        row[42:46] if row[39:42] == '000' else row[39:46],
                    ),
                    # 'remote_account_name': row[],
                    'amount': decimal.Decimal('%s%s.%s' % ('-' if row[46] == '0' else '', row[50:63], row[63:65])),
                    'variable_symbol': int(row[127:137]),
                    'constant_symbol': int(row[137:147]),
                    'specific_symbol': int(row[147:157]),
                    'sender_description': row[269:409].strip(),
                    'recipient_description': row[209:239].strip(),
                }



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin

from .models import Order, OrderPayment


@admin.register(Order)
class OrderAdmin(admin.ModelAdmin):
    list_display = ('name', 'variable_symbol',)


@admin.register(OrderPayment)
class OrderPaymentAdmin(admin.ModelAdmin):
    list_display = ('order', 'transaction', 'amount')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-10-30 23:00
from __future__ import unicode_literals

from bankreader.readers import get_reader_choices
from django.db import migrations, models
import django.db.models.deletion


def set_transaction_account_statement(apps, schema_editor):
    AccountStatement = apps.get_model('bankreader', 'AccountStatement')
    Transaction = apps.get_model('bankreader', 'Transaction')
    for transaction in Transaction.objects.iterator():
        transaction.account_statement = (
            AccountStatement.objects.filter(
                from_date__lte=transaction.accounted_date,
                to_date__gte=transaction.accounted_date,
            )
            .order_by('id')
            .first()
        )
        if transaction.account_statement:
            transaction.save()
    orphans = Transaction.objects.filter(account_statement=None)
    first, last = orphans.first(), orphans.last()
    if first and last:
        orphans.update(
            account_statement=AccountStatement.objects.create(
                account_id=transaction.account_id,
                statement='_deleted_',
                from_date=first.accounted_date,
                to_date=last.accounted_date,
            )
        )


class Migration(migrations.Migration):

    dependencies = [
        ('bankreader', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='accountstatement',
            options={
                'ordering': ('from_date',),
                'verbose_name': 'account statement',
                'verbose_name_plural': 'account statements',
            },
        ),
        migrations.AlterField(
            model_name='accountstatement',
            name='account',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='account_statements',
                to='bankreader.Account',
                verbose_name='account',
            ),
        ),
        migrations.AddField(
            model_name='account',
            name='reader',
            field=models.CharField(
                blank=True,
                choices=get_reader_choices(),
                max_length=150,
                null=True,
                verbose_name='account statement format',
            ),
        ),
        migrations.AddField(
            model_name='transaction',
            name='account_statement',
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name='transactions',
                to='bankreader.AccountStatement',
                verbose_name='account statement',
            ),
        ),
        migrations.RunPython(set_transaction_account_statement),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-10-30 23:18
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bankreader', '0002_transaction_account_statement'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transaction',
            name='account_statement',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='transactions',
                to='bankreader.AccountStatement',
                verbose_name='account statement',
            ),
        ),
    ]



import re
from mt940.parser import parse as mt940_parse
from typing import Iterable

from .base import BaseReader


class MT940Reader(BaseReader):
    label = 'MT940 (MultiCash)'

    def read_transactions(self, rows: Iterable[str]):
        for transaction in mt940_parse(''.join(rows)):
            purpose = transaction.data.get("purpose")
            symbols = dict(re.findall(r"([KVS]S) ([0-9]{10})", purpose))
            account_match = re.search(r"([0-9]+-)?([0-9]{10})/([0-9]{4})", purpose)
            description = re.split("([KVS]S [0-9]{10}){3}", purpose)[-1]
            yield {
                "transaction_id": transaction.data.get("customer_reference"),
                "entry_date": transaction.data.get("entry_date"),
                "accounted_date": transaction.data.get("date"),
                "remote_account_number": account_match and account_match.group(),
                "remote_account_name": transaction.data.get("applicant_name"),
                "amount": transaction.data.get("amount").amount,
                "variable_symbol": symbols.get("VS"),
                "constant_symbol": symbols.get("KS"),
                "specific_symbol": symbols.get("SS"),
                "sender_description": description,
                "recipient_description": description,
            }



import datetime

from .base import BaseReader


class GpcReader(BaseReader):
    label = 'GPC'

    def read_transactions(self, rows):
        transaction = None
        for row in rows:
            # first row of transaction data
            if row[:3] == '075':
                if transaction:
                    # send previous transaction data
                    if 'entry_date' not in transaction:
                        transaction['entry_date'] = transaction['accounted_date']
                    yield transaction
                    transaction = None
                if row[60] in '12':
                    # create new transaction data
                    amount = int(row[48:60]) / 100.0
                    transaction = {
                        'transaction_id': row[35:48],
                        'accounted_date': datetime.datetime.strptime(row[122:128], '%d%m%y').date(),
                        'remote_account_number': '%s-%s/%s'
                        % (
                            row[19:25],
                            row[25:35],
                            row[73:77],
                        ),
                        'remote_account_name': row[97:117].strip(),
                        'amount': amount if row[60] == '2' else -amount,
                        'variable_symbol': int(row[61:71]),
                        'constant_symbol': int(row[77:81]),
                        'specific_symbol': int(row[81:91]),
                    }
            # second row of transaction data
            elif transaction and row[:3] == '076':
                try:
                    transaction['entry_date'] = datetime.datetime.strptime(row[29:35], '%d%m%y').date()
                except ValueError:
                    transaction['entry_date'] = transaction['accounted_date']
                transaction['sender_description'] = row[35:127].strip()
            # third row of transaction data
            elif transaction and row[:3] == '078':
                transaction['recipient_description'] = row[3:127].strip()
            # 4th row of transaction data
            elif transaction and row[:3] == '079':
                transaction['recipient_description'] += row[3:73].strip()
        if transaction:
            if 'entry_date' not in transaction:
                transaction['entry_date'] = transaction['accounted_date']
            yield transaction



# Generated by Django 2.1.1 on 2018-10-02 09:36

from django.db import migrations, models
import django.db.models.deletion
import localflavor.generic.models

from bankreader.readers import get_reader_choices


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, unique=True, verbose_name='account name')),
                (
                    'iban',
                    localflavor.generic.models.IBANField(
                        blank=True,
                        include_countries=None,
                        max_length=34,
                        null=True,
                        use_nordea_extensions=False,
                        verbose_name='IBAN',
                    ),
                ),
                (
                    'bic',
                    localflavor.generic.models.BICField(
                        blank=True, max_length=11, null=True, verbose_name='BIC (SWIFT)'
                    ),
                ),
            ],
            options={
                'verbose_name': 'account',
                'verbose_name_plural': 'accounts',
            },
        ),
        migrations.CreateModel(
            name='AccountStatement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('statement', models.CharField(max_length=256, verbose_name='statement')),
                ('from_date', models.DateField(editable=False, verbose_name='from date')),
                ('to_date', models.DateField(editable=False, verbose_name='to date')),
                (
                    'account',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to='bankreader.Account', verbose_name='account'
                    ),
                ),
            ],
            options={
                'ordering': ('from_date',),
            },
        ),
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('transaction_id', models.CharField(max_length=256, verbose_name='transaction id')),
                ('entry_date', models.DateField(verbose_name='entry date')),
                ('accounted_date', models.DateField(verbose_name='accounted date')),
                ('remote_account_number', models.CharField(max_length=64, verbose_name='remote account number')),
                ('remote_account_name', models.CharField(max_length=128, verbose_name='remote account name')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=20, verbose_name='amount')),
                ('variable_symbol', models.BigIntegerField(default=0, verbose_name='variable symbol')),
                ('constant_symbol', models.BigIntegerField(default=0, verbose_name='constant symbol')),
                ('specific_symbol', models.BigIntegerField(default=0, verbose_name='specific symbol')),
                ('sender_description', models.CharField(max_length=256, verbose_name='description for sender')),
                ('recipient_description', models.CharField(max_length=256, verbose_name='description for recipient')),
                (
                    'account',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to='bankreader.Account', verbose_name='account'
                    ),
                ),
            ],
            options={
                'verbose_name': 'transaction',
                'verbose_name_plural': 'transactions',
                'ordering': ('accounted_date',),
            },
        ),
        migrations.AlterUniqueTogether(
            name='transaction',
            unique_together={('account', 'transaction_id')},
        ),
    ]



from bankreader.readers import register_reader
from bankreader.readers.best import BestReader
from bankreader.readers.csv import CsvReader


@register_reader
class KbBestReader(BestReader):
    label = 'Komerční banka Best'


@register_reader
class KbCsvReader(CsvReader):
    label = 'Komerční banka CSV'
    encoding = 'cp1250'
    delimiter = ';'
    column_mapping = {
        'Datum splatnosti': 'accounted_date',
        'Datum odepsani JB': 'entry_date',
        'Protiucet/Kod banky': 'remote_account_number',
        'Nazev protiuctu': 'remote_account_name',
        'Castka': 'amount',
        'VS': 'variable_symbol',
        'KS': 'constant_symbol',
        'SS': 'specific_symbol',
        'Identifikace transakce': 'transaction_id',
        'Popis prikazce': 'sender_description',
        'Popis pro prijemce': 'recipient_description',
    }
    date_format = '%Y-%m-%d'
    decimal_separator = ','



import io
from zipfile import BadZipFile, ZipFile


class BaseReader:
    encoding = 'utf-8'

    @property
    def label(self):
        raise NotImplementedError()

    def read_archive(self, statement_file):
        """Try to unpack ZIP archive and call read() for each file."""
        try:
            zip_file = ZipFile(statement_file)
        except BadZipFile:
            try:
                statement_file.seek(0)
            except io.UnsupportedOperation:
                pass
            for transaction in self.read_transactions(self._decode(statement_file)):
                yield transaction
        else:
            for zip_info in zip_file.filelist:
                with zip_file.open(zip_info) as f:
                    for transaction in self.read_transactions(self._decode(f)):
                        yield transaction

    def _decode(self, rows):
        return (row.decode(self.encoding) if type(row) == bytes else row for row in rows)

    def read_transactions(self, rows):
        raise NotImplementedError()



import csv
import datetime
import decimal
import re

from .base import BaseReader


class CsvReader(BaseReader):
    label = 'CSV'
    column_mapping = {}
    date_format = '%Y-%m-%d'
    delimiter = ','
    quotechar = '"'
    encoding = 'utf-8'
    decimal_separator = '.'
    decimalregex = re.compile(r'[^0-9,-]')

    def __init__(self):
        self.decimal_cleaner = re.compile(r'[^0-9-%s]' % self.decimal_separator)

    def read_transactions(self, rows):
        column_mapping = {}
        csv_reader = csv.reader(rows, delimiter=self.delimiter, quotechar=self.quotechar)
        for row in csv_reader:
            if not column_mapping:
                try:
                    column_mapping = {key: row.index(csv_key) for csv_key, key in self.column_mapping.items()}
                except ValueError:
                    pass
                continue
            yield {key: self.get_value(key, row[column_mapping[key]]) for key in column_mapping}

    def get_value(self, key, value):
        if key in ('accounted_date', 'entry_date'):
            return datetime.datetime.strptime(value, self.date_format).date()
        elif key == 'amount':
            return decimal.Decimal(self.decimal_cleaner.sub('', value).replace(self.decimal_separator, '.'))
        elif key.endswith('_symbol'):
            return int(value) if value.isdigit() else 0
        else:
            return value



readers = {}


def get_reader_choices():
    return sorted([(key, reader.label) for key, reader in readers.items()], key=lambda x: x[1])


def register_reader(reader):
    readers[getattr(reader, 'key', '%s.%s' % (reader.__module__, reader.__name__))] = reader()
    return reader
